import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.util.HashMap;

import javax.swing.*;

import acm.graphics.*;
import acm.program.GraphicsProgram;

public class BoxDiagram extends GraphicsProgram { 
	static final long serialVersionUID = 1;

	public void init() { 
		contents = new HashMap<String,GObject>(); 
		createController(); 
		addActionListeners(); 
		addMouseListeners(); 
	} 

	private void createController() { 
		nameField = new JTextField(MAX_NAME); 
		nameField.addActionListener(this); 
		nameField.setActionCommand("Add"); 
		add(new JLabel("Name"), SOUTH); 
		add(nameField, SOUTH); 
		add(new JButton("Add"), SOUTH); 
		add(new JButton("Remove"), SOUTH); 
		add(new JButton("Clear"), SOUTH); 
	} 

	private void addBox(String name) { 
		GCompound box = new GCompound(); 
		GRect outline = new GRect(BOX_WIDTH, BOX_HEIGHT); 
		GLabel label = new GLabel(name); 
		box.add(outline, -BOX_WIDTH / 2, -BOX_HEIGHT / 2); 
		box.add(label, -label.getWidth()/2, label.getAscent()/2); 
		add(box, getWidth() / 2, getHeight() / 2); 
		contents.put(name, box); 
	} 

	private void removeBox(String name) { 
		GObject obj = contents.get(name); 
		if (obj != null) { 
			remove(obj); 
			contents.remove(name); 
		}
	} 

	private void clear() { 
		removeAll(); 
		contents.clear(); 
		nameField.setText(""); 
	} 

	public void actionPerformed(ActionEvent e) { 
		String cmd = e.getActionCommand(); 
		if (cmd.equals("Add")) { 
			addBox(nameField.getText()); 
		} else if (cmd.equals("Remove")) { 
			removeBox(nameField.getText()); 
		} else if (cmd.equals("Clear")) { 
			clear(); 
		} 
	} 

public void mousePressed(MouseEvent e) { 
	last = new GPoint(e.getPoint()); 
	currentObject = getElementAt(last); 
	} 

public void mouseDragged(MouseEvent e) { 
	if (currentObject != null) { 
		currentObject.move(e.getX() - last.getX(), 
                   e.getY() - last.getY()); 
		last = new GPoint(e.getPoint()); 
	} 
} 

public void mouseClicked(MouseEvent e) { 
	if (currentObject != null) currentObject.sendToFront(); 
} 

private static final int MAX_NAME = 25; 
private static final double BOX_WIDTH = 120; 
private static final double BOX_HEIGHT = 50; 

private HashMap<String,GObject> contents; 
private JTextField nameField; 
private GObject currentObject; 
private GPoint last; 
}
